@@ -38,24 +38,24 @@ static inline bool isWPEKeyCodeFromKeyPad(unsigned keyCode)
     return keyCode >= WPE_KEY_KP_Space && keyCode <= WPE_KEY_KP_9;
 }
 
-static OptionSet<WebEvent::Modifier> modifiersForEventModifiers(unsigned eventModifiers)
+static OptionSet<WebEventModifier> modifiersForEventModifiers(unsigned eventModifiers)
 {
-    OptionSet<WebEvent::Modifier> modifiers;
+    OptionSet<WebEventModifier> modifiers;
     if (eventModifiers & wpe_input_keyboard_modifier_control)
-        modifiers.add(WebEvent::Modifier::ControlKey);
+        modifiers.add(WebEventModifier::ControlKey);
     if (eventModifiers & wpe_input_keyboard_modifier_shift)
-        modifiers.add(WebEvent::Modifier::ShiftKey);
+        modifiers.add(WebEventModifier::ShiftKey);
     if (eventModifiers & wpe_input_keyboard_modifier_alt)
-        modifiers.add(WebEvent::Modifier::AltKey);
+        modifiers.add(WebEventModifier::AltKey);
     if (eventModifiers & wpe_input_keyboard_modifier_meta)
-        modifiers.add(WebEvent::Modifier::MetaKey);
+        modifiers.add(WebEventModifier::MetaKey);
 
     return modifiers;
 }
 
-static OptionSet<WebEvent::Modifier> modifiersForKeyboardEvent(struct wpe_input_keyboard_event* event)
+static OptionSet<WebEventModifier> modifiersForKeyboardEvent(struct wpe_input_keyboard_event* event)
 {
-    OptionSet<WebEvent::Modifier> modifiers = modifiersForEventModifiers(event->modifiers);
+    OptionSet<WebEventModifier> modifiers = modifiersForEventModifiers(event->modifiers);
 
     if (!event->pressed)
         return modifiers;
@@ -66,22 +66,22 @@ static OptionSet<WebEvent::Modifier> modifiersForKeyboardEvent(struct wpe_input_
     switch (event->key_code) {
     case WPE_KEY_Control_L:
     case WPE_KEY_Control_R:
-        modifiers.add(WebEvent::Modifier::ControlKey);
+        modifiers.add(WebEventModifier::ControlKey);
         break;
     case WPE_KEY_Shift_L:
     case WPE_KEY_Shift_R:
-        modifiers.add(WebEvent::Modifier::ShiftKey);
+        modifiers.add(WebEventModifier::ShiftKey);
         break;
     case WPE_KEY_Alt_L:
     case WPE_KEY_Alt_R:
-        modifiers.add(WebEvent::Modifier::AltKey);
+        modifiers.add(WebEventModifier::AltKey);
         break;
     case WPE_KEY_Meta_L:
     case WPE_KEY_Meta_R:
-        modifiers.add(WebEvent::Modifier::MetaKey);
+        modifiers.add(WebEventModifier::MetaKey);
         break;
     case WPE_KEY_Caps_Lock:
-        modifiers.add(WebEvent::Modifier::CapsLockKey);
+        modifiers.add(WebEventModifier::CapsLockKey);
         break;
     }
 
@@ -113,9 +113,9 @@ WallTime wallTimeForEventTime(uint64_t msTimeStamp)
     return firstEventWallTime + Seconds(delta / 1000.);
 }
 
-WebKeyboardEvent WebEventFactory::createWebKeyboardEvent(struct wpe_input_keyboard_event* event, const String& text, bool handledByInputMethod, std::optional<Vector<WebCore::CompositionUnderline>>&& preeditUnderlines, std::optional<EditingRange>&& preeditSelectionRange)
+WebKeyboardEvent WebEventFactory::createWebKeyboardEvent(struct wpe_input_keyboard_event* event, const String& text, bool isAutoRepeat, bool handledByInputMethod, std::optional<Vector<WebCore::CompositionUnderline>>&& preeditUnderlines, std::optional<EditingRange>&& preeditSelectionRange)
 {
-    return WebKeyboardEvent(event->pressed ? WebEvent::KeyDown : WebEvent::KeyUp,
+    return WebKeyboardEvent({ event->pressed ? WebEventType::KeyDown : WebEventType::KeyUp, modifiersForKeyboardEvent(event), wallTimeForEventTime(event->time) },
         text.isNull() ? WebCore::PlatformKeyboardEvent::singleCharacterString(event->key_code) : text,
         WebCore::PlatformKeyboardEvent::keyValueForWPEKeyCode(event->key_code),
         WebCore::PlatformKeyboardEvent::keyCodeForHardwareKeyCode(event->hardware_key_code),
@@ -125,9 +125,9 @@ WebKeyboardEvent WebEventFactory::createWebKeyboardEvent(struct wpe_input_keyboa
         handledByInputMethod,
         WTFMove(preeditUnderlines),
         WTFMove(preeditSelectionRange),
-        isWPEKeyCodeFromKeyPad(event->key_code),
-        modifiersForKeyboardEvent(event),
-        wallTimeForEventTime(event->time));
+        isAutoRepeat,
+        isWPEKeyCodeFromKeyPad(event->key_code)
+        );
 }
 
 static inline short pressedMouseButtons(uint32_t modifiers)
@@ -155,42 +155,67 @@ static inline short pressedMouseButtons(uint32_t modifiers)
     return buttons;
 }
 
-WebMouseEvent WebEventFactory::createWebMouseEvent(struct wpe_input_pointer_event* event, float deviceScaleFactor)
+static inline unsigned clickCount(struct wpe_input_pointer_event* event)
 {
-    WebEvent::Type type = WebEvent::NoType;
+    static const uint32_t doubleClickTime = 500; // Milliseconds.
+    static const int pixelThreshold = 5;
+    static struct wpe_input_pointer_event gLastClickEvent = { };
+    static unsigned gLastClickCount = 1;
+
+    bool cancelPreviousClick = (event->time - gLastClickEvent.time) > doubleClickTime
+        || std::abs(event->x - gLastClickEvent.x) > pixelThreshold
+        || std::abs(event->y - gLastClickEvent.y) > pixelThreshold;
+
+    if (event->type == wpe_input_pointer_event_type_button && event->state) {
+        if (!cancelPreviousClick && (event->button == gLastClickEvent.button))
+            ++gLastClickCount;
+        else
+            gLastClickCount = 1;
+        gLastClickEvent = *event;
+    } else if (event->type == wpe_input_pointer_event_type_motion) {
+        if (cancelPreviousClick) {
+            gLastClickCount = 0;
+            gLastClickEvent = { };
+        }
+    }
+
+    return gLastClickCount;
+}
+
+WebMouseEvent WebEventFactory::createWebMouseEvent(struct wpe_input_pointer_event* event, float deviceScaleFactor, WebMouseEventSyntheticClickType syntheticClickType)
+{
+    auto type = WebEventType::MouseMove;
     switch (event->type) {
     case wpe_input_pointer_event_type_motion:
-        type = WebEvent::MouseMove;
+        type = WebEventType::MouseMove;
         break;
     case wpe_input_pointer_event_type_button:
-        type = event->state ? WebEvent::MouseDown : WebEvent::MouseUp;
+        type = event->state ? WebEventType::MouseDown : WebEventType::MouseUp;
         break;
     case wpe_input_pointer_event_type_null:
         ASSERT_NOT_REACHED();
     }
 
-    WebMouseEvent::Button button = WebMouseEvent::NoButton;
+    WebMouseEventButton button = WebMouseEventButton::None;
     switch (event->type) {
     case wpe_input_pointer_event_type_motion:
     case wpe_input_pointer_event_type_button:
         if (event->button == 1)
-            button = WebMouseEvent::LeftButton;
+            button = WebMouseEventButton::Left;
         else if (event->button == 2)
-            button = WebMouseEvent::RightButton;
+            button = WebMouseEventButton::Right;
         else if (event->button == 3)
-            button = WebMouseEvent::MiddleButton;
+            button = WebMouseEventButton::Middle;
         break;
     case wpe_input_pointer_event_type_null:
         ASSERT_NOT_REACHED();
     }
 
-    unsigned clickCount = (type == WebEvent::MouseDown) ? 1 : 0;
-
-    // FIXME: Proper button support. deltaX/Y/Z. Click count.
+    // FIXME: Proper button support. deltaX/Y/Z.
     WebCore::IntPoint position(event->x, event->y);
     position.scale(1 / deviceScaleFactor);
-    return WebMouseEvent(type, button, pressedMouseButtons(event->modifiers), position, position,
-        0, 0, 0, clickCount, modifiersForEventModifiers(event->modifiers), wallTimeForEventTime(event->time));
+    return WebMouseEvent({ type, modifiersForEventModifiers(event->modifiers), wallTimeForEventTime(event->time) }, button, pressedMouseButtons(event->modifiers), position, position,
+        0, 0, 0, clickCount(event), 0, syntheticClickType);
 }
 
 WebWheelEvent WebEventFactory::createWebWheelEvent(struct wpe_input_axis_event* event, float deviceScaleFactor, WebWheelEvent::Phase phase, WebWheelEvent::Phase momentumPhase)
@@ -217,13 +242,12 @@ WebWheelEvent WebEventFactory::createWebWheelEvent(struct wpe_input_axis_event*
             hasPreciseScrollingDeltas = true;
             break;
         default:
-            return WebWheelEvent();
+            ASSERT_NOT_REACHED();
         }
 
-        return WebWheelEvent(WebEvent::Wheel, position, position,
-            delta, wheelTicks, phase, momentumPhase,
-            WebWheelEvent::ScrollByPixelWheelEvent, hasPreciseScrollingDeltas,
-            OptionSet<WebEvent::Modifier> { }, wallTimeForEventTime(event->time));
+        return WebWheelEvent({ WebEventType::Wheel, OptionSet<WebEventModifier> { }, wallTimeForEventTime(event->time) }, position, position,
+            delta, wheelTicks, WebWheelEvent::ScrollByPixelWheelEvent, phase, momentumPhase,
+            hasPreciseScrollingDeltas);
     }
 #endif
 
@@ -251,51 +275,51 @@ WebWheelEvent WebEventFactory::createWebWheelEvent(struct wpe_input_axis_event*
         hasPreciseScrollingDeltas = true;
         break;
     default:
-        return WebWheelEvent();
+        ASSERT_NOT_REACHED();
     };
 
-    return WebWheelEvent(WebEvent::Wheel, position, position,
-        delta, wheelTicks, phase, momentumPhase,
-        WebWheelEvent::ScrollByPixelWheelEvent, hasPreciseScrollingDeltas,
-        OptionSet<WebEvent::Modifier> { }, wallTimeForEventTime(event->time));
+    return WebWheelEvent({ WebEventType::Wheel, OptionSet<WebEventModifier> { }, wallTimeForEventTime(event->time) }, position, position,
+        delta, wheelTicks, WebWheelEvent::ScrollByPixelWheelEvent, phase, momentumPhase,
+        hasPreciseScrollingDeltas);
 }
 
 #if ENABLE(TOUCH_EVENTS)
-static WebKit::WebPlatformTouchPoint::TouchPointState stateForTouchPoint(int mainEventId, const struct wpe_input_touch_event_raw* point)
+static WebKit::WebPlatformTouchPoint::State stateForTouchPoint(int mainEventId, const struct wpe_input_touch_event_raw* point)
 {
     if (point->id != mainEventId)
-        return WebKit::WebPlatformTouchPoint::TouchStationary;
+        return WebKit::WebPlatformTouchPoint::State::Stationary;
 
     switch (point->type) {
     case wpe_input_touch_event_type_down:
-        return WebKit::WebPlatformTouchPoint::TouchPressed;
+        return WebKit::WebPlatformTouchPoint::State::Pressed;
     case wpe_input_touch_event_type_motion:
-        return WebKit::WebPlatformTouchPoint::TouchMoved;
+        return WebKit::WebPlatformTouchPoint::State::Moved;
     case wpe_input_touch_event_type_up:
-        return WebKit::WebPlatformTouchPoint::TouchReleased;
+        return WebKit::WebPlatformTouchPoint::State::Released;
     case wpe_input_touch_event_type_null:
         ASSERT_NOT_REACHED();
         break;
     };
 
-    return WebKit::WebPlatformTouchPoint::TouchStationary;
+    return WebKit::WebPlatformTouchPoint::State::Stationary;
 }
 
 WebTouchEvent WebEventFactory::createWebTouchEvent(struct wpe_input_touch_event* event, float deviceScaleFactor)
 {
-    WebEvent::Type type = WebEvent::NoType;
+    WebEventType type;
     switch (event->type) {
     case wpe_input_touch_event_type_down:
-        type = WebEvent::TouchStart;
+        type = WebEventType::TouchStart;
         break;
     case wpe_input_touch_event_type_motion:
-        type = WebEvent::TouchMove;
+        type = WebEventType::TouchMove;
         break;
     case wpe_input_touch_event_type_up:
-        type = WebEvent::TouchEnd;
+        type = WebEventType::TouchEnd;
         break;
-    case wpe_input_touch_event_type_null:
+    default:
         ASSERT_NOT_REACHED();
+        break;
     }
 
     Vector<WebKit::WebPlatformTouchPoint> touchPoints;
@@ -308,12 +332,12 @@ WebTouchEvent WebEventFactory::createWebTouchEvent(struct wpe_input_touch_event*
 
         WebCore::IntPoint pointCoordinates(point.x, point.y);
         pointCoordinates.scale(1 / deviceScaleFactor);
-        touchPoints.uncheckedAppend(
+        touchPoints.append(
             WebKit::WebPlatformTouchPoint(point.id, stateForTouchPoint(event->id, &point),
                 pointCoordinates, pointCoordinates));
     }
 
-    return WebTouchEvent(type, WTFMove(touchPoints), OptionSet<WebEvent::Modifier> { }, wallTimeForEventTime(event->time));
+    return WebTouchEvent({ type, OptionSet<WebEventModifier> { }, wallTimeForEventTime(event->time) }, WTFMove(touchPoints), { }, { });
 }
 #endif // ENABLE(TOUCH_EVENTS)
 
